import json

import pytest

from mythx_models.exceptions import ValidationError
from mythx_models.response import AuthRefreshResponse

from . import common as testdata

ACCESS_TOKEN = "my_fancy_access_token"
REFRESH_TOKEN = "my_fancy_refresh_token"
AUTH_REFRESH = {"access": ACCESS_TOKEN, "refresh": REFRESH_TOKEN}
AUTH_REFRESH_RESPONSE = AuthRefreshResponse(
    access_token=ACCESS_TOKEN, refresh_token=REFRESH_TOKEN
)


def assert_auth_refresh_response(resp: AuthRefreshResponse):
    assert resp.access_token == testdata.ACCESS_TOKEN_1
    assert resp.refresh_token == testdata.REFRESH_TOKEN_1


def test_auth_refresh_response_from_valid_json():
    resp = AuthRefreshResponse.from_json(json.dumps(testdata.REFRESH_RESPONSE_DICT))
    assert_auth_refresh_response(resp)


def test_auth_refresh_response_from_invalid_json():
    with pytest.raises(ValidationError):
        AuthRefreshResponse.from_json("{}")


def test_auth_refresh_response_from_valid_dict():
    resp = AuthRefreshResponse.from_dict(testdata.REFRESH_RESPONSE_DICT)
    assert_auth_refresh_response(resp)


def test_auth_refresh_response_from_invalid_dict():
    with pytest.raises(ValidationError):
        AuthRefreshResponse.from_dict({})


def test_auth_refresh_response_to_json():
    assert (
        json.loads(testdata.REFRESH_RESPONSE_OBJECT.to_json())
        == testdata.REFRESH_RESPONSE_DICT
    )


def test_auth_refresh_response_to_dict():
    assert testdata.REFRESH_RESPONSE_OBJECT.to_dict() == testdata.REFRESH_RESPONSE_DICT



import json

import pytest

from mythx_models.exceptions import ValidationError
from mythx_models.response import OASResponse

from . import common as testdata


def test_oas_response_from_valid_json():
    resp = OASResponse.from_json(testdata.OPENAPI_RESPONSE)
    assert resp.data == testdata.OPENAPI_RESPONSE


def test_oas_response_from_valid_dict():
    resp = OASResponse.from_dict({"data": testdata.OPENAPI_RESPONSE})
    assert resp.data == testdata.OPENAPI_RESPONSE


def test_oas_response_from_invalid_dict():
    with pytest.raises(ValidationError):
        OASResponse.from_dict({})


def test_oas_response_invalid_type():
    with pytest.raises(TypeError):
        OASResponse(data=1)


def test_oas_response_to_json():
    assert testdata.OPENAPI_RESPONSE_OBJECT.to_json() == json.dumps(
        {"data": testdata.OPENAPI_RESPONSE}
    )


def test_oas_response_to_dict():
    assert testdata.OPENAPI_RESPONSE_OBJECT.to_dict() == {
        "data": testdata.OPENAPI_RESPONSE
    }



import json

import pytest

from mythx_models.exceptions import ValidationError
from mythx_models.response import VersionResponse

from . import common as testdata


def assert_version_response(resp: VersionResponse):
    assert resp.api_version == testdata.API_VERSION_1
    assert resp.maru_version == testdata.MARU_VERSION_1
    assert resp.mythril_version == testdata.MYTHRIL_VERSION_1
    assert resp.harvey_version == testdata.HARVEY_VERSION_1
    assert resp.hashed_version == testdata.HASHED_VERSION_1


def test_auth_logout_request_from_valid_json():
    resp = VersionResponse.from_json(json.dumps(testdata.VERSION_RESPONSE_DICT))
    assert_version_response(resp)


def test_auth_logout_request_from_valid_dict():
    resp = VersionResponse.from_dict(testdata.VERSION_RESPONSE_DICT)
    assert_version_response(resp)


def test_auth_logout_request_from_invalid_dict():
    with pytest.raises(ValidationError):
        VersionResponse.from_dict({})


def test_auth_logout_request_from_invalid_json():
    with pytest.raises(ValidationError):
        VersionResponse.from_json("{}")


def test_auth_logout_request_to_json():
    assert (
        json.loads(testdata.VERSION_RESPONSE_OBJECT.to_json())
        == testdata.VERSION_RESPONSE_DICT
    )


def test_auth_logout_request_to_dict():
    assert testdata.VERSION_RESPONSE_OBJECT.to_dict() == testdata.VERSION_RESPONSE_DICT



import json
from copy import deepcopy

from mythx_models.response import (
    Issue,
    Severity,
    SourceFormat,
    SourceLocation,
    SourceType,
)

from . import common as testdata


def assert_issue(issue: Issue, skip_decoded=False):
    assert issue.swc_id == testdata.SWC_ID
    assert issue.swc_title == testdata.SWC_TITLE
    assert issue.description_short == testdata.DESCRIPTION_HEAD
    assert issue.description_long == testdata.DESCRIPTION_TAIL
    assert issue.severity == Severity(testdata.SEVERITY)
    if not skip_decoded:
        assert issue.decoded_locations == testdata.DECODED_LOCATIONS_OBJ
    assert len(issue.locations) == 1
    location = issue.locations[0]
    assert location.source_map.to_sourcemap() == testdata.SOURCE_MAP
    assert location.source_format == SourceFormat.EVM_BYZANTIUM_BYTECODE
    assert location.source_type == SourceType.RAW_BYTECODE
    assert location.source_list == testdata.SOURCE_LIST


def test_issue_from_valid_json():
    issue = Issue.from_json(json.dumps(testdata.ISSUE_DICT))
    assert_issue(issue)


def test_issue_from_dict():
    issue = Issue.from_dict(testdata.ISSUE_DICT)
    assert_issue(issue)


def test_issue_to_json():
    assert json.loads(testdata.ISSUE_OBJECT.to_json()) == testdata.ISSUE_DICT


def test_issue_to_dict():
    assert testdata.ISSUE_OBJECT.to_dict() == testdata.ISSUE_DICT


def test_source_location_from_dict():
    sl = SourceLocation.from_dict(testdata.SOURCE_LOCATION)
    assert sl.source_format == testdata.SOURCE_FORMAT
    assert sl.source_list == testdata.SOURCE_LIST
    assert sl.source_map.to_sourcemap() == testdata.SOURCE_MAP
    assert sl.source_type == testdata.SOURCE_TYPE


def test_decoded_locations_removed():
    issue_data = deepcopy(testdata.ISSUE_DICT)
    del issue_data["decodedLocations"]
    issue = Issue.from_dict(issue_data)
    assert_issue(issue, skip_decoded=True)
    assert "decodedLocations" not in issue.to_dict()


def test_decoded_locations_empty_removed():
    issue_data = deepcopy(testdata.ISSUE_DICT)
    issue_data["decodedLocations"] = []
    issue = Issue.from_dict(issue_data)
    assert_issue(issue, skip_decoded=True)
    assert "decodedLocations" not in issue.to_dict()


def test_decoded_locations_only_removed():
    issue_data = deepcopy(testdata.ISSUE_DICT)
    issue_data["decodedLocations"] = [[]]
    issue = Issue.from_dict(issue_data)
    assert_issue(issue, skip_decoded=True)
    assert "decodedLocations" not in issue.to_dict()


def test_decoded_locations_empty_skip():
    issue_data = deepcopy(testdata.ISSUE_DICT)
    issue_data["decodedLocations"].append([])
    issue = Issue.from_dict(issue_data)
    assert_issue(issue, skip_decoded=True)
    assert "decodedLocations" in issue.to_dict()



import json

import pytest

from mythx_models.exceptions import ValidationError
from mythx_models.response import AuthLoginResponse

from . import common as testdata


def assert_auth_login_response(resp: AuthLoginResponse):
    assert resp.access_token == testdata.ACCESS_TOKEN_1
    assert resp.refresh_token == testdata.REFRESH_TOKEN_1


def test_auth_login_response_from_valid_json():
    resp = AuthLoginResponse.from_json(json.dumps(testdata.LOGIN_RESPONSE_DICT))
    assert_auth_login_response(resp)


def test_auth_login_response_from_invalid_json():
    with pytest.raises(ValidationError):
        AuthLoginResponse.from_json("{}")


def test_auth_login_response_from_valid_dict():
    resp = AuthLoginResponse.from_dict(testdata.LOGIN_RESPONSE_DICT)
    assert_auth_login_response(resp)


def test_auth_login_response_from_invalid_dict():
    with pytest.raises(ValidationError):
        AuthLoginResponse.from_dict({})


def test_auth_login_response_to_json():
    assert (
        json.loads(testdata.LOGIN_RESPONSE_OBJECT.to_json())
        == testdata.LOGIN_RESPONSE_DICT
    )


def test_auth_login_response_to_dict():
    assert testdata.LOGIN_RESPONSE_OBJECT.to_dict() == testdata.LOGIN_RESPONSE_DICT



import pytest

from mythx_models import request as reqmodels
from mythx_models import response as respmodels
from mythx_models.exceptions import ValidationError

from . import common as testdata


@pytest.mark.parametrize(
    "r_dict,model,should_raise,exc",
    [
        (
            testdata.ANALYSIS_LIST_REQUEST_DICT,
            reqmodels.AnalysisListRequest,
            False,
            ValidationError,
        ),
        (
            testdata.DETECTED_ISSUES_REQUEST_DICT,
            reqmodels.DetectedIssuesRequest,
            False,
            ValidationError,
        ),
        (
            testdata.ANALYSIS_STATUS_REQUEST_DICT,
            reqmodels.AnalysisStatusRequest,
            False,
            ValidationError,
        ),
        (
            testdata.ANALYSIS_SUBMISSION_REQUEST_DICT,
            reqmodels.AnalysisSubmissionRequest,
            True,
            ValidationError,
        ),
        (
            testdata.LOGIN_REQUEST_DICT,
            reqmodels.AuthLoginRequest,
            True,
            ValidationError,
        ),
        (
            testdata.LOGOUT_REQUEST_DICT,
            reqmodels.AuthLogoutRequest,
            True,
            ValidationError,
        ),
        (
            testdata.REFRESH_REQUEST_DICT,
            reqmodels.AuthRefreshRequest,
            True,
            ValidationError,
        ),
        (
            testdata.ANALYSIS_LIST_RESPONSE_DICT,
            respmodels.AnalysisListResponse,
            True,
            ValidationError,
        ),
        (
            testdata.DETECTED_ISSUES_RESPONSE_DICT,
            respmodels.DetectedIssuesResponse,
            True,
            ValidationError,
        ),
        (
            testdata.ANALYSIS_STATUS_RESPONSE_DICT,
            respmodels.AnalysisStatusResponse,
            True,
            ValidationError,
        ),
        (
            testdata.ANALYSIS_SUBMISSION_RESPONSE_DICT,
            respmodels.AnalysisSubmissionResponse,
            True,
            ValidationError,
        ),
        (
            testdata.LOGIN_RESPONSE_DICT,
            respmodels.AuthLoginResponse,
            True,
            ValidationError,
        ),
        (
            testdata.LOGOUT_RESPONSE_DICT,
            respmodels.AuthLogoutResponse,
            False,
            ValidationError,
        ),
        (
            testdata.REFRESH_RESPONSE_DICT,
            respmodels.AuthRefreshResponse,
            True,
            ValidationError,
        ),
    ],
)
def test_model_validators(r_dict, model, should_raise, exc):
    if should_raise:
        with pytest.raises(exc):
            model.validate({})
    else:
        model.validate(r_dict)



import json

import pytest

from mythx_models.exceptions import ValidationError
from mythx_models.request import AuthLogoutRequest

from . import common as testdata


def assert_logout_request(req):
    assert req.global_ == testdata.GLOBAL_LOGOUT
    assert req.method == "POST"
    assert req.headers == {}
    assert req.parameters == {}
    assert req.payload == {}


def test_auth_logout_request_from_valid_json():
    req = AuthLogoutRequest.from_json(json.dumps(testdata.LOGOUT_REQUEST_DICT))
    assert_logout_request(req)


def test_auth_logout_request_from_invalid_json():
    with pytest.raises(ValidationError):
        AuthLogoutRequest.from_json("{}")


def test_auth_logout_request_from_valid_dict():
    req = AuthLogoutRequest.from_dict(testdata.LOGOUT_REQUEST_DICT)
    assert_logout_request(req)


def test_auth_logout_request_from_invalid_dict():
    with pytest.raises(ValidationError):
        AuthLogoutRequest.from_dict({})


def test_auth_logout_request_to_json():
    assert (
        json.loads(testdata.LOGOUT_REQUEST_OBJECT.to_json())
        == testdata.LOGOUT_REQUEST_DICT
    )


def test_auth_logout_request_to_dict():
    assert testdata.LOGOUT_REQUEST_OBJECT.to_dict() == testdata.LOGOUT_REQUEST_DICT



import pytest

from mythx_models.exceptions import ValidationError
from mythx_models.response import AuthLogoutResponse

from . import common as testdata


def test_auth_login_response_from_valid_json():
    resp = AuthLogoutResponse.from_json("{}")
    assert type(resp) == AuthLogoutResponse


def test_auth_login_response_from_invalid_json():
    with pytest.raises(ValidationError):
        AuthLogoutResponse.from_json('{"foo": "bar"}')


def test_auth_login_response_from_valid_dict():
    resp = AuthLogoutResponse.from_dict({})
    assert type(resp) == AuthLogoutResponse


def test_auth_login_response_from_invalid_dict():
    with pytest.raises(ValidationError):
        AuthLogoutResponse.from_dict({"foo": "bar"})


def test_auth_login_response_to_json():
    assert testdata.LOGOUT_RESPONSE_OBJECT.to_json() == "{}"


def test_auth_login_response_to_dict():
    assert testdata.LOGOUT_RESPONSE_OBJECT.to_dict() == {}



import json
from copy import copy

import dateutil.parser

from mythx_models.response import Analysis, AnalysisStatus

from . import common as testdata


def assert_analysis(analysis):
    assert analysis.uuid == testdata.UUID_1
    assert analysis.api_version == testdata.API_VERSION_1
    assert analysis.maru_version == testdata.MARU_VERSION_1
    assert analysis.mythril_version == testdata.MYTHRIL_VERSION_1
    assert analysis.harvey_version == testdata.HARVEY_VERSION_1
    assert analysis.queue_time == testdata.QUEUE_TIME_1
    assert analysis.run_time == 0  # default value
    assert analysis.status == AnalysisStatus(testdata.STATUS_1)
    assert analysis.submitted_at == dateutil.parser.parse(testdata.SUBMITTED_AT_1)
    assert analysis.submitted_by == testdata.SUBMITTED_BY_1


def test_analysis_from_valid_json():
    analysis = Analysis.from_json(json.dumps(testdata.ANALYSIS_DICT))
    assert_analysis(analysis)


def test_analysis_to_json():
    assert json.loads(testdata.ANALYSIS_OBJECT.to_json()) == testdata.ANALYSIS_DICT


def test_analysis_to_dict():
    assert testdata.ANALYSIS_OBJECT.to_dict() == testdata.ANALYSIS_DICT


def test_analysis_propagate_error_field():
    analysis = copy(testdata.ANALYSIS_OBJECT)
    # add optional error field
    analysis.error = testdata.ERROR
    analysis_dict = analysis.to_dict()
    analysis_dict["error"] == testdata.ERROR


def test_analysis_from_valid_dict():
    analysis = Analysis.from_dict(testdata.ANALYSIS_DICT)
    assert_analysis(analysis)


def test_repr():
    analysis_repr = repr(testdata.ANALYSIS_OBJECT)
    assert testdata.ANALYSIS_OBJECT.uuid in analysis_repr
    testdata.ANALYSIS_OBJECT.status in analysis_repr



from mythx_models.request.analysis_status import AnalysisStatusRequest


class AnalysisInputRequest(AnalysisStatusRequest):
    @property
    def endpoint(self):
        """The API's analysis request input endpoint.

        :return: A string denoting the request input endpoint without the host prefix
        """
        return "v1/analyses/{}/input".format(self.uuid)



import pytest

from mythx_models.exceptions import ValidationError
from mythx_models.request import OASRequest

from . import common as testdata


def assert_version_request(req):
    assert req.method == "GET"
    assert req.headers == {}
    assert req.parameters == {}
    assert req.payload == {}
    if req.mode == "html":
        assert req.endpoint == "v1/openapi"
    else:
        assert req.endpoint == "v1/openapi.yaml"


def test_oas_request_from_valid_json():
    req = OASRequest.from_json("{}")
    assert_version_request(req)


def test_oas_request_from_valid_dict():
    req = OASRequest.from_dict({})
    assert_version_request(req)


def test_invalid_format():
    with pytest.raises(ValidationError):
        OASRequest(mode="invalid")


def test_oas_request_to_json():
    assert testdata.OPENAPI_REQUEST_OBJECT.to_json() == "{}"


def test_oas_request_to_dict():
    assert testdata.OPENAPI_REQUEST_OBJECT.to_dict() == {}
